import getpass
import sys
import subprocess
from curtains.state import env


__all__  = ["win_command_loop"]


def win_make_commands(commands):
    # get smarter about it
    return [([c['cmd'] for c in commands], True)]


def win_command_loop(commands, username=None, password=None, hosts=None):
    username = username or env.get("username")
    password = password or env.get("password")
    hosts = hosts or env.get("hosts")
    command_batches = win_make_commands(commands)

    auth_cmd = win_get_credential(username, password)
    for cmd_list, remote in command_batches:
        if remote:
            processes = []
            for cmpt_name in hosts:
                # cmpt_name = % i
                invoke_cmd = win_invoke_command(cmpt_name, cmd_list)
                cmd = auth_cmd + invoke_cmd
                print("Executing on machine %s" % cmpt_name)
                proc = subprocess.Popen(["powershell.exe", "-ExecutionPolicy",
                                         'RemoteSigned', "-Command",
                                         ";".join(cmd)
                                         ],
                                        stdout=sys.stdout, stderr=sys.stderr)
                processes.append(proc)
            for proc in processes:
                out, err = proc.communicate()


def win_invoke_command(computer_name, remote_commands):
    remote_script = ' "&" '.join(remote_commands)
    #cmd = ['Invoke-Command -computername %s -credential $cred -ScriptBlock { & cmd.exe /c %s }' % \
    cmd = ['Invoke-Command -computername %s -ScriptBlock { & cmd.exe /c %s }' % (computer_name, remote_script)]
    return cmd


def win_get_credential(username=None, password=None):
    current_user = getpass.getuser()
    username = username or current_user
    stripped_user = username.split("\\")[-1]
    need_auth = stripped_user != current_user
    if (password is None) and need_auth:
        password = getpass.getpass("Provide password for %s: " % username)

    cmd = []
    if need_auth:
        cmd = ['$passwd=convertto-securestring -AsPlainText -Force -String "%s"' % password,
               '$cred=new-object -typename System.Management.Automation.PSCredential -argumentlist "%s", $passwd' % username]
    return []




__all__ = ['run', 'local']


class Command(object):
    _cmd = []

    @classmethod
    def clear_commands(cls):
        cls._cmd.clear()

    @classmethod
    def commands(cls):
        return cls._cmd


class Run(Command):
    def __call__(self, cmd):
        self._cmd.append({'cmd': cmd, 'type': 'batch', 'class': 'Run'})


class Local(Command):
    def __call__(self, cmd):
        self._cmd.append({'cmd':cmd, 'type': 'batch', 'class': 'Local'})


def XCopy(Command):
    def __call__(self, localpath, remotepath):
        pass


run = Run()
local = Local()


from functools import wraps
from curtains.commands import Command

class task(object):
    _task_list = []

    def __init__(self, username=None, password=None, hosts=None, ):
        self.hosts = hosts
        self.username = username
        self.password = password


    def __call__(self, fn):
        self._task_list.append({'hosts': self.hosts, 'username': self.username, 'password':self.password, 'fn': fn})
        @wraps(fn)
        def decorated(*args, **kwargs):
            out = fn(args, kwargs)
            return out
        return decorated

    @classmethod
    def task_list(cls):
        return cls._task_list




import os
import click
import inspect
from functools import wraps
from curtains.commands import Command
from curtains.osspecific import win_command_loop
from curtains import state


@click.group(chain=True)
@click.pass_context
def cli(ctx):
    pass


@cli.resultcallback()
def process_result(result, **kwargs):
    pass


def cmd_loop(commands, env):
    if os.name == 'nt':
        win_command_loop(commands, **env)
    else:
        raise NotImplementedError("Unsupported OS type "+os.name)


def executor(fn, env):
    @wraps(fn)
    def decorated(*args, **kwargs):
        Command.clear_commands()
        fn(*args, **kwargs)
        cmd_loop(Command.commands(), env)
    return decorated


def create_cli():
    from curtains.decorator import task
    task_list = task.task_list()
    #task_fn = [t['fn'], t) for t in task_list]
    for task in task_list:
        t = task.pop('fn')
        argnames = [str(a) for a in inspect.signature(t).parameters.keys()]
        fn = executor(t, task)
        for arg in argnames[::-1]:
            click.argument(arg)(fn)
        cli.command()(fn)
    return cli

def _is_package(path):
    """
    Is the given path a Python package?
    """
    _exists = lambda s: os.path.exists(os.path.join(path, s))
    return (
        os.path.isdir(path)
        and (_exists('__init__.py') or _exists('__init__.pyc'))
    )


def find_curtfile(names=None):
    """
    Attempt to locate a fabfile, either explicitly or by searching parent dirs.
    Usage docs are in sites/docs/usage/fabfiles.rst, in "Fabfile discovery."
    """
    # Obtain env value if not given specifically
    if names is None:
        names = [state.env["curtfile"]]
    # Create .py version if necessary
    if not names[0].endswith('.py'):
        names += [names[0] + '.py']
    # Does the name contain path elements?
    if os.path.dirname(names[0]):
        # If so, expand home-directory markers and test for existence
        for name in names:
            expanded = os.path.expanduser(name)
            if os.path.exists(expanded):
                if name.endswith('.py') or _is_package(expanded):
                    return os.path.abspath(expanded)
    else:
        # Otherwise, start in cwd and work downwards towards filesystem root
        path = '.'
        # Stop before falling off root of filesystem (should be platform
        # agnostic)
        while os.path.split(os.path.abspath(path))[1]:
            for name in names:
                joined = os.path.join(path, name)
                if os.path.exists(joined):
                    if name.endswith('.py') or _is_package(joined):
                        return os.path.abspath(joined)
            path = os.path.join('..', path)


def main(curtfile_locations=None):
    curtfile = find_curtfile(curtfile_locations)
    print(curtfile)
    exec(open(curtfile).read())
    cli = create_cli()
    cli()


if __name__ == '__main__':
    main(os.path.join(os.getcwd(), "curtfile.py"))


"""
Curtains
--------
Curtains is a command line tool for remote execution, application deployment and system administration
in the Windows environment (as of now) without using SSH.
"""

from setuptools import setup, find_packages
import os
import re
import ast

_version_re = re.compile(r'__version__\s+=\s+(.*)')

with open('curtains/__init__.py', 'rb') as f:
    version = str(ast.literal_eval(_version_re.search(
        f.read().decode('utf-8')).group(1)))



BASE_PATH = os.path.dirname(__file__)


def get_requirements(suffix=''):
    with open(os.path.join(BASE_PATH, 'Requirements%s.txt' % suffix)) as f:
        rv = f.read().splitlines()
    return rv

setup(
    name='Curtains',
    version=version,
    url='https://github.com/gouthambs/Curtains',
    license='MIT',
    author='Gouthaman Balaraman',
    author_email='gouthaman.balaraman@gmail.com',
    description=' A command line tool for remote execution, application deployment and system '
                'administration in the Windows environment (as of now) without using SSH.',
    long_description=__doc__,
    packages=find_packages(),
    zip_safe=False,
    include_package_data=True,
    platforms='win',
    install_requires=get_requirements(),
    tests_require=[],
    entry_points={
        'console_scripts': [
            'curt = curtains.main:main',
        ]
    },
    classifiers=[
        'Development Status :: 4 - Beta',
        'Environment :: Console',
        'Intended Audience :: Developers',
        'Intended Audience :: System Administrators',
        'License :: OSI Approved :: MIT License',
        'Operating System :: Microsoft',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.5',
        'Topic :: Software Development',
        'Topic :: Software Development :: Build Tools',
        'Topic :: Software Development :: Libraries',
        'Topic :: Software Development :: Libraries :: Python Modules',
        'Topic :: System :: Clustering',
        'Topic :: System :: Software Distribution',
        'Topic :: System :: Systems Administration'
    ]
)